package Aww::lib;

use strict;
use warnings;

use Config::Simple ;
use File::Copy ;
use Crypt::CBC ;
use Image::Magick;
use File::Basename;

use Aww::Options ;


require Exporter ;

our @ISA = qw(Exporter) ;

our @EXPORT_OK = qw(admininput checkoptions checksite checktime createthumb config connect createthumb execoptions get_filenames get_fullfilenames getsubdirs image_properties match optioninput pft2oft print_error process trashfiles wmlprocess writemainconfig) ;



=pod

=head1 NAME

B<Aww::lib> - Perl module for aww

=head2 EXPORT_OK

admininput checkoptions checksite checktime config
execoptions get_filenames get_fullfilenames getsubdirs match
optioninput print_error process trashfiles wmlprocess writemainconfig


=head1 DESCRIPTION 

This is the function library for the aww program. That's why it
exports (if you want it) a lot of functions. This library comes in
handy if you want to program your own modules for aww. You will find
here a short description for every supplied function. How you call it
(IN) and what you will get back (OUT).


=cut


####################################################################
# admininput subroutine, to add key-valuepairs to a hash

=pod

=head2 admininput

This subroutine adds key-valuepairs to the project configfile:

=over 7

=item B<IN : ><the new key>

=back

The subroutine then asks for the value of the new key, chomps of the linebreak, applies some sanity checks and hands it over to the config subroutine.

=over 7

=item B<OUT: >New element in project configfile

=back

=cut

 
sub admininput {
    my $NewFile = shift ;
    my $NewKey = shift ;
    my $KeyValue;

    chomp ($KeyValue = <STDIN>);

    # Remove any trailing slash
    unless ( $KeyValue =~ m{^/$} ) {
	$KeyValue =~ s{/$}{} ;
		   }

    # Check for ; and space
    if ( $KeyValue =~ /;/ ) { print_error("No semikolon allowed for $NewKey!")}
    if ( $KeyValue =~ / / ) { print_error("No space allowed for $NewKey!")}

    # Many more sanity checks are needed here!

     if ( $NewKey =~ m/^Export_FTP_Password$/ ) {

	 if ( $KeyValue ) {
	 # encrypt the password:
	 my $cp = Crypt::CBC->new( {'key'   => 'HaCKerZANzwer',
				    'cipher' => 'Blowfish' }) ;

	 $KeyValue = $cp->encrypt($KeyValue) ;
         }
     }


    $NewKey = "Main.$NewKey" ;

    config($NewFile,$NewKey,$KeyValue) ;


}
	
 





####################################################################
# checkoptions subroutine

=pod

=head2 checkoptions

This subroutine checks if the options that are given are OK (if they
are in 'PossibleOptions' from the config file)

=over 7

=item B<IN : > [all options for the directory]

=back

=cut



sub checkoptions {
    my @Options = @_ ;


    print "-- executing checkoptions subroutine --\n" if $Aww::verbose > 1 ;

    print "Options to check: @Options\n" if $Aww::verbose > 2;

    my @PossibleOptions = config("config","PossibleOptions") ;

OPTION: foreach my $In ( @Options ) {
          foreach my $Out ( @PossibleOptions ) {

	      next OPTION if $In =~ /^$Out$/ ;

	  }
	  die "'$In' Option not possible!\nOnly '@PossibleOptions' Options are possible!\n";
      }
}







####################################################################
# checksite subroutine

=pod

=head2 checksite

This subroutine checks if the projects exist that were given on the
commandline.

=over 7

=item B<IN : >All projects given on the commandline, or 'all'

=item B<OUT: >Only the existing projects (of the given), or 'all' projects

=back

=cut


sub checksite {
    print "-- executing checksite subroutine --\n" if $Aww::verbose > 1 ;
    my @CheckSites = @_;
    my @SitesPresent;
    print "Checking the following Sites: @CheckSites \n" if $Aww::verbose > 1 ;
    opendir (AWWCONFIGDIR, "$ENV{HOME}/.aww") ;
    my @AllSites =  map { /(.*)\.aww$/ } readdir AWWCONFIGDIR;
    closedir AWWCONFIGDIR;
    print "Aww-Configfiles for the following sites exist: @AllSites \n" if $Aww::verbose > 1 ;

    # Return AllSites if checked for all
    if ( grep { /^all$/ } @CheckSites ) {
	return @AllSites ;
    }
    # Otherwise only return the present sites
    else {
	my $CheckSites;
	foreach $CheckSites (@CheckSites) {
	    if ( grep { /^$CheckSites$/ } @AllSites ) {
		push @SitesPresent, $CheckSites ;
	    }

	}
	return @SitesPresent ;
    }
}





####################################################################
# checktime subroutine

=pod

=head2 checktime

This subroutine checks if destination files to the given source files
exist and if they are newer (if they are up to date).

The subroutine checks in the destination directory for the same
filename and for a filename with Outputfile_type suffix

If the --force flag is used, the checktime subroutine returns all
files that have destination files older than the start of the program.

=over 7

=item B<IN : >Sourcefiles to test

=item B<OUT: >Sourcefiles that don't have a newer Destination-file 
-> which have to be processed

=back

=cut



sub checktime{
    my @Sourcefiles = @_ ;
    my @Processfiles ;

    my $Linkfile = "" ;

    print "-- executing checktime subroutine --\n" if $Aww::verbose > 1;

    if ( $Aww::force ) {

	print "   Entering force mode\n"  if $Aww::verbose > 1 ;

      FORCE: foreach my $Sourcefile ( @Sourcefiles ) {
	  
	  if ( -r $Sourcefile ) {

	      $Linkfile = "" ;

	      # For symbolic links, check the file, that is linked to:
	      if ( -l $Sourcefile ) {

		  $Linkfile = $Sourcefile ;
		  $Sourcefile = readlink $Sourcefile  ;
		  my $SourceDirname = dirname($Linkfile) ;
		  $Sourcefile = qq($SourceDirname/$Sourcefile) unless $Sourcefile =~ m{^/} ;
	      }

	      my $Testfile = $Sourcefile ;
	      $Testfile =~ s(^$Aww::Main::ProcessVars->{'Main.Src_Base_Directory'})($Aww::Main::ProcessVars->{'Main.Output_Base_Directory'}) ;
	      
	      print "Checking time for $Testfile\n" if $Aww::verbose > 2 ;
	      
	      if ( -r $Testfile ) {
		  if ( -M $Testfile < 0 ) {		   
		      print " -> Just created\n"  if $Aww::verbose > 2 ;
		      next FORCE;
		  }
	      }
	      my $OFT = config("config","Outputfile_type") ;
	      no warnings ;
	      $OFT = eval "\"$OFT\"" ;
	      $Testfile =~ s/\.\w*$/$OFT/ ;
	      print "Checking time for $Testfile\n" if $Aww::verbose > 2 ;

	      if ( -r $Testfile ) {
		  if ( -M $Testfile < 0 ) {		    
		      print " -> Just created\n"  if $Aww::verbose > 2 ;
		      next FORCE;
		  }
	      }
	      
	      print " -> Not newly created! Adding to processfiles\n"  if $Aww::verbose > 2 ;

	      $Sourcefile = $Linkfile if -l $Linkfile ;

	      push @Processfiles , $Sourcefile ;		       
	      
	  }  
      }

    }
    else {

      SRC: foreach my $Sourcefile ( @Sourcefiles ) {
	  
	  if ( -r $Sourcefile ) {

	      $Linkfile = "" ;

	      # For symbolic links, check the file, that is linked to:
	      if ( -l $Sourcefile ) {

		  $Linkfile = $Sourcefile ;
		  $Sourcefile = readlink $Sourcefile  ;
		  my $SourceDirname = dirname($Linkfile) ;
		  $Sourcefile = qq($SourceDirname/$Sourcefile) unless $Sourcefile =~ m{^/} ;
	      }
	      
	      my $Testfile = $Sourcefile ;
	      $Testfile =~ s(^$Aww::Main::ProcessVars->{'Main.Src_Base_Directory'})($Aww::Main::ProcessVars->{'Main.Output_Base_Directory'}) ;

	      unless ( $Aww::Main::ProcessVars->{'Main.Src_Base_Directory'} eq $Aww::Main::ProcessVars->{'Main.Output_Base_Directory'} ) {
	      
		  print "Checking time for $Testfile\n" if $Aww::verbose > 2 ;
	      
		  if ( -r $Testfile ) {
		      if ( -M $Testfile <= -M $Sourcefile ) {		   
			  print " -> Up to date\n"  if $Aww::verbose > 2 ;
			  next SRC;
		      }
		  }
	      }

	      my $OFT = config("config","Outputfile_type") ;
	      no warnings ;
	      $OFT = eval "\"$OFT\"" ;
	      $Testfile =~ s/\.\w*$/$OFT/ ;
	      print "Checking time for $Testfile\n" if $Aww::verbose > 2 ;
	      
	      if ( -r $Testfile ) {
		  if ( -M $Testfile <= -M $Sourcefile ) {		    
		      print " -> Up to date\n"  if $Aww::verbose > 2 ;
		      next SRC;
		  }
	      }
	      
	      print " -> Old! Adding to processfiles\n"  if $Aww::verbose > 2 ;

	      $Sourcefile = $Linkfile if -l $Linkfile ;

	      push @Processfiles , $Sourcefile ;		       
	      
	  }  
      }
    }

    print "processfiles: @Processfiles\n" if $Aww::verbose > 1 ;
    return @Processfiles ;


}




####################################################################
# config subroutine

=pod

=head2 config

This subroutine is used for all configurations in aww. 
(replacing all old configuration subroutines)


=over 7

=item B<IN : ><config-file>, [parameter to return the value], [input value to write to parameter]

=item B<OUT: >depends on the input

=over 

=item 1. true or false if given config file is readable

=item 2. the value of the parameter that was asked for 

=item 3. true (the written value) or false if the new value was
successfully written to the config file

=back

=back

=cut




sub config {
    my $ConfigFile = shift ;
    my $Parameter = shift ;
    my $NewValue = shift ;

    print "-- executing config subroutine --\n" if $Aww::verbose > 2 ;

    # If the file is not given with the full path assume standard .aww
    # directory!

    unless ( $ConfigFile =~ m{^/} ) { 

	$ConfigFile = "$ENV{HOME}/.aww/$ConfigFile" ;

    }

    my $Config ;

    if ( -r $ConfigFile ) {

	$Config = new Config::Simple($ConfigFile) or die Config::Simple->error() ;

    }
    else {

	$Config = new Config::Simple(syntax=>'ini') or die Config::Simple->error() ;
    }


    # if $NewValue exist, write new value and return the written value
    
    if ( $NewValue ) {

	$Config->param($Parameter,$NewValue) ;
	$Config->write($ConfigFile) ;
	return "$Parameter=$NewValue" ;
	
    }

    # if $Parameter exist, return the value of the parameter
    elsif ( $Parameter ) {
	
	return $Config->param($Parameter) ;
	
    }
    
    # Otherwise return all variables in the configfile as a
    # reference to a hash
    
    elsif ( $ConfigFile ) {
	
	my %AllVars = $Config->vars() ;
	return \%AllVars ;
	
    }
    
    else {
    
	return 0 ;
    
    }


}





####################################################################
# connect subroutine

=pod

=head2 connect

The connect subroutine processes the given files of the process
directory with WML-Options to connect to Previous, Up, Next pages etc.

The WML-Options can be accessed in the wml files with <get-var VARIABLENAME>:
PREVIOUSFILE, NEXTFILE and UPFILE

=over 7

=item B<In :> <Process Directory>,<UpOption - none|alpha|one|a number>,<UpFile Basename>,<@FilesToConnect>


=over

=item * none: don't use an UpOption

=item * one: put all Files on one indexpage (for a small number of files)

=item * alpha|reverse: For alphabetical index pages (a: abla, ablubb and b: bla, blubb and c: cfile, canoterfile) and z to a for reverse, of course

=item * a number: number or entries per index page

=back


=item B<Out:> executing the process-function with the right
WML-Options(e.g. -DPREVIOUSFILE -DNEXTFILE -DUPFILE)

=back

=cut





sub connect {
    my $SrcDir = shift ;
    my $UpOption = shift ;
    my $UpFileBasename = shift ;
    my @ConnectFiles = @_ ;

    print "-- executing Aww::Options::connect subroutine --\n" if $Aww::verbose > 2 ;

    my $DestDir = $SrcDir;
    $DestDir =~ s/^$Aww::Main::ProcessVars->{'Main.Src_Base_Directory'}/$Aww::Main::ProcessVars->{'Main.Output_Base_Directory'}/ ;


#    my $PFT = config("config","Processfile_type") ;
    my $OFT = config("config","Outputfile_type") ;
    no warnings ;
    $OFT = eval "\"$OFT\"" ;

    if ( $UpOption eq 'none' ) {

	print "   using UpOption none\n" if $Aww::verbose > 2 ;

	foreach my $i ( 0 .. $#ConnectFiles ) {

	    my $FullSrcFile = "$SrcDir/$ConnectFiles[$i]" ;
	    my $FullDestFile = "$DestDir/$ConnectFiles[$i]" ;
	    $FullDestFile = pft2oft($FullDestFile) ;          # change ending

	    my $PreviousFile = "" ;

	    unless ( $i-1 < 0 ){
		$PreviousFile = $ConnectFiles[$i-1] ;
		$PreviousFile = pft2oft($PreviousFile) ;      # change ending
	    }

	    my $PreviousVar = qq(-DPREVIOUSFILE=$PreviousFile) ;
	

	    my $NextFile = "" ;
	    if ( $ConnectFiles[$i+1] ) {
		$NextFile = $ConnectFiles[$i+1] ;
		$NextFile = pft2oft($NextFile) ;          # change ending
	    }

	    my $NextVar = qq(-DNEXTFILE=$NextFile) ;
	

	    my @WMLOption = ( $PreviousVar,$NextVar ) ;

	    Aww::lib::process($FullSrcFile,$FullDestFile,@WMLOption);

	}

    }
    elsif ( $UpOption eq 'one' ) {

	print "   using UpOption one\n" if $Aww::verbose > 2 ;

	my $UpFile = "" ;
	$UpFile = "$UpFileBasename$OFT" ;
	my $UpVar = qq(-DUPFILE=$UpFile) ;


	foreach my $i ( 0 .. $#ConnectFiles ) {

	    my $FullSrcFile = "$SrcDir/$ConnectFiles[$i]" ;
	    my $FullDestFile = "$DestDir/$ConnectFiles[$i]" ;
	    $FullDestFile = pft2oft($FullDestFile) ;          # change ending

	    my $PreviousFile = "" ;

	    unless ( $i-1 < 0 ){
		$PreviousFile = $ConnectFiles[$i-1] ;
		$PreviousFile = pft2oft($PreviousFile) ;      # change ending
	    }

	    my $PreviousVar = qq(-DPREVIOUSFILE=$PreviousFile) ;
	

	    my $NextFile = "" ;
	    if ( $ConnectFiles[$i+1] ) {
		$NextFile = $ConnectFiles[$i+1] ;
		$NextFile = pft2oft($NextFile) ;          # change ending
	    }

	    my $NextVar = qq(-DNEXTFILE=$NextFile) ;


	    my @WMLOption = ( $PreviousVar,$UpVar,$NextVar ) ;

	    Aww::lib::process($FullSrcFile,$FullDestFile,@WMLOption);

	}

    }
    elsif ( $UpOption =~ m/^alpha$|^reverse$/ ) {

	print "   using UpOption alpha\n" if $Aww::verbose > 2 ;


	# all INDEX Variables are always given: 
	my @UpVars = () ;
	my @UpFiles = ("","b".."z") ;
	foreach my $UpFile ( @UpFiles ) {
	    my $BigFile = uc($UpFile) ;
	    push @UpVars , qq(-DINDEX$BigFile=$UpFileBasename$UpFile$OFT ) ;
	}


	LOOP: foreach my $i ( 0 .. $#ConnectFiles ) {

	    my $FullSrcFile = "$SrcDir/$ConnectFiles[$i]" ;
	    my $FullDestFile = "$DestDir/$ConnectFiles[$i]" ;
	    $FullDestFile = pft2oft($FullDestFile) ;          # change ending

	    my $PreviousFile = "" ;

	    unless ( $i-1 < 0 ){
		$PreviousFile = $ConnectFiles[$i-1] ;
		$PreviousFile = pft2oft($PreviousFile) ;      # change ending
	    }

	    my $PreviousVar = qq(-DPREVIOUSFILE=$PreviousFile) ;
	

	    my $NextFile = "" ;
	    if ( $ConnectFiles[$i+1] ) {
		$NextFile = $ConnectFiles[$i+1] ;
		$NextFile = pft2oft($NextFile) ;          # change ending
	    }

	    my $NextVar = qq(-DNEXTFILE=$NextFile) ;


	    my $FirstLetter = substr($ConnectFiles[$i], 0 , 1) ;
	    $FirstLetter = lc($FirstLetter) ;
	    $FirstLetter =~ tr/a-z//dc ;
	    $FirstLetter =~ tr/a//d if $UpOption eq 'alpha' ;
	    $FirstLetter =~ tr/z//d if $UpOption eq 'reverse' ;

	    my $UpVar = qq(-DUPFILE=$UpFileBasename$FirstLetter$OFT) ;



	    my @WMLOption = ( $PreviousVar,@UpVars,$UpVar,$NextVar ) ;

	    Aww::lib::process($FullSrcFile,$FullDestFile,@WMLOption);

	}

    }
    elsif ( $UpOption =~ m/^\d+$/ ) {

	print "   using UpOption number $UpOption\n" if $Aww::verbose > 2 ;

	my $FilesPerPage = $UpOption ;
	my $Number = "" ;

	foreach my $i ( 0 .. $#ConnectFiles ) {

	    my $FullSrcFile = "$SrcDir/$ConnectFiles[$i]" ;
	    my $FullDestFile = "$DestDir/$ConnectFiles[$i]" ;
	    $FullDestFile = pft2oft($FullDestFile) ;          # change ending

	    my $PreviousFile = "" ;

	    unless ( $i-1 < 0 ){
		$PreviousFile = $ConnectFiles[$i-1] ;
		$PreviousFile = pft2oft($PreviousFile) ;      # change ending
	    }

	    my $PreviousVar = qq(-DPREVIOUSFILE=$PreviousFile) ;
	

	    my $NextFile = "" ;
	    if ( $ConnectFiles[$i+1] ) {
		$NextFile = $ConnectFiles[$i+1] ;
		$NextFile = pft2oft($NextFile) ;          # change ending
	    }

	    my $NextVar = qq(-DNEXTFILE=$NextFile) ;


	    if ( $i == $FilesPerPage ) {
		$FilesPerPage = $FilesPerPage + $UpOption ;
		$Number++ ;
		$Number++ if $Number == 1 ;
	    }

	    my $UpVar = qq(-DUPFILE=$UpFileBasename$Number$OFT) ;


	    my @WMLOption = ( $PreviousVar,$UpVar,$NextVar ) ;

	    Aww::lib::process($FullSrcFile,$FullDestFile,@WMLOption);

	}

    }
    else {

	print_error("UpOption '$UpOption' in connect module not known") ;

    }

}



####################################################################
# createthumb subroutine

=pod

=head2 createthumb

This subroutine creates a thumbnail with the given options.

=over 7

=item B<IN : ><SourceDir>, <thumbtype>, <thumbdir>, <maxwidth in px>, <maxheight in px>, <@SrcImges> 

=item B<OUT: > created thumbs , @Thumbpaths

=back

=cut


sub createthumb {
    my $SrcDir = shift ;
    my $ThumbType = shift ;
    my $ThumbBaseDir = shift ;
    my $maxwidth = shift ;
    my $maxheight = shift ;
    my @SrcImages = @_ ;

    my @Thumbs ;
    print "-- executing createthumb subroutine --\n" if $Aww::verbose > 1 ;

    my $geometry = "${maxwidth}x${maxheight}" ;

    my $ThumbDir = "$SrcDir/$ThumbBaseDir" ;
    $ThumbDir =~ s(^$Aww::Main::ProcessVars->{'Main.Src_Base_Directory'})($Aww::Main::ProcessVars->{'Main.Output_Base_Directory'}) ;
    mkdir $ThumbDir ;

    foreach my $SrcImage ( @SrcImages ) {

	my $DestImage = "$ThumbDir/$SrcImage" ;
	$DestImage =~ s(\.\w*$)(\.$ThumbType) ;

	my $Thumbnail = new Image::Magick;
	
	my $ThumbStatus = $Thumbnail->Read("$SrcDir/$SrcImage") ;
	print_error($ThumbStatus) if $ThumbStatus ;
	
	$ThumbStatus = $Thumbnail->Resize(geometry=>$geometry) ;
	print_error($ThumbStatus) if $ThumbStatus ;

	$ThumbStatus = $Thumbnail->Write("$ThumbType:$DestImage") ;
	print_error($ThumbStatus) if $ThumbStatus ;

	push @Thumbs , $DestImage ;

	print "   creating thumb: $DestImage\n" if $Aww::verbose > 2 ; 

    }

    return @Thumbs ;
}













####################################################################
# execoptions subroutine

=pod

=head2 execoptions

This subroutine executes the functions for all options (one after another). 

=over 7

=item B<IN : ><directory>, [all options of the directory]

=item B<OUT: >the option-module-function is called with the directory
name as argument

=back

=cut


sub execoptions {
    my $OptionDir = shift;
    my @Options = @_ ;

    print "-- executing execoptions subroutine --\n" if $Aww::verbose > 1 ;

    print "Options for $OptionDir: @Options\n" if $Aww::verbose > 1;

    foreach my $Option ( @Options ) {
	no strict 'refs' ; # Without this line you can't call &{$FullOption}
	my $FullOption = "Aww::Options::$Option";
	&{$FullOption} ($OptionDir) ;
    }


}






####################################################################
# get_filenames subroutine

=pod

=head2 get_filenames

This subroutine returns all filenames of given directory that have a
special ending.

=over 7

=item B<IN : ><directory>, <sort options>, [RegExp]

=item B<OUT: >Sorted list of filenames that matched the RegExp (ignoring the case)

=back

=cut


sub get_filenames {
    my $DirGet = shift ;
    my $SortOption = shift ;
    my $MatchRegExp = shift ;


    my @AllFilesInDir ;
    my @FilesInDir ;

    opendir(DIRGET, "$DirGet") or die "Couldn't open $DirGet: $!\n";
    @AllFilesInDir = readdir DIRGET ;
    closedir DIRGET ;

    if ( $MatchRegExp ) {

	print "Getting /${MatchRegExp}/-files of Directory $DirGet : \n" if $Aww::verbose > 1 ;


	foreach ( @AllFilesInDir ) {
	    push @FilesInDir, $_ if ( $_ =~ /$MatchRegExp/i );
	}	    
    }
    else {
	print "Getting all files of Directory $DirGet : \nOnly removing hidden '.*' files\n" if $Aww::verbose > 1 ;

	my $StripHidden = "^[^\.]" ;

	foreach ( @AllFilesInDir ) {
	    push @FilesInDir, $_ if ( $_ =~ /$StripHidden/ );
	}


    }

    if ( $SortOption =~/^alpha$/ ) {

	print "Alpanumeric sorting (case insensitive) of filenames \n" if $Aww::verbose > 1 ;
	@FilesInDir = sort { lc($a) cmp lc($b) } @FilesInDir ;


    }
    elsif ( $SortOption =~/^Alpha$/ ) {

	print "Alpanumeric sorting (case sensitive) of filenames \n" if $Aww::verbose > 1 ;
	@FilesInDir = sort @FilesInDir ;
    }
    elsif ( $SortOption =~/^reverse$/ ) {

	print "Reverse alpanumeric sorting (case insensitive) of filenames \n" if $Aww::verbose > 1 ;
	@FilesInDir = sort { lc($b) cmp lc($a) } @FilesInDir ;


    }
    elsif ( $SortOption =~/^Reverse$/ ) {

	print "Reverse alpanumeric sorting (case sensitive) of filenames \n" if $Aww::verbose > 1 ;
	@FilesInDir = sort { $b cmp $a } @FilesInDir ;

    }
    elsif ( $SortOption =~/^time$/ ) {

	print "Time sorting (newest first) of filenames \n" if $Aww::verbose > 1 ;
	# using the orcish maneuver for sorting:
	my %m ; 
	@FilesInDir = sort { ($m{$a} ||= -M $a) <=> ($m{$b} ||= -M $b) } @FilesInDir;

	

    }
    elsif ( $SortOption =~/^Time$/ ) {

	print "Time sorting (oldest first) of filenames \n" if $Aww::verbose > 1 ;

	# using the orcish maneuver for sorting:
	my %m ; 
	@FilesInDir = sort { ($m{$b} ||= -M $b) <=> ($m{$a} ||= -M $a) } @FilesInDir;


    }
    elsif ( $SortOption =~/^none$/ ) {

	print "No sorting of filenames \n" if $Aww::verbose > 1 ;

    }
    else {

	print_error("Wrong sort option for get_filenames subroutine") ;

    }




    return @FilesInDir ;
}










####################################################################
# get_fullfilenames subroutine

=pod

=head2 get_fullfilenames

This subroutine gets all filenames of given directory that match the
given RegExp and sorts them after the sortoption. The subroutines
calls get_filenames and adds the full path to the files!

=over 7

=item B<IN : ><directory>, <sort option>, [RegExpe]

=item B<OUT: >Sorted list of full filenames with the FileType ending

=back

=cut


sub get_fullfilenames {
    my $DirGet = shift ;
    my $SortOption = shift ;
    my $MatchOption = shift ;

    my @Files = get_filenames($DirGet,$SortOption,$MatchOption) ;



    foreach my $FileInDir ( @Files ) {

	$FileInDir = qq($DirGet/$FileInDir) ;

    }


    return @Files ;
}











####################################################################
# getsubdirs subroutine:
# (gets recursive all directory names of Startdir)

=pod

=head2 getsubdirs

This subroutine gets recursivly all subdirectories of given directories

=over 7

=item B<IN : ><directories>

=item B<OUT: >All Subdirectories of the given directories including themselves

=back

=cut



sub getsubdirs {
    my @NewDir = @_; 

    my @AllDirs = @NewDir ;
    while ( @NewDir ) {
	my $NewDir = shift @NewDir ;
	opendir(NEWDIR, $NewDir) or die "Couldn't open Directory $NewDir!\n";
	my @NewFiles = grep !/^\./, readdir NEWDIR ;
	my @Dirs;
	
	foreach (@NewFiles) {
	    @Dirs = (@Dirs, "$NewDir/$_") if -d "$NewDir/$_";
	}
	
	print "Directories in $NewDir: @Dirs\n" if $Aww::verbose > 1 ;
	
	closedir NEWDIR ;
	push @AllDirs, @Dirs ;
	push @NewDir, @Dirs ;
	

    }


    return @AllDirs;

}







####################################################################
# image_properties subroutine:
# (gets recursive all directory names of Startdir)

=pod

=head2 image_properties

This subroutine returns the following properties of the input image: width, height, size

=over 7

=item B<IN : ><imagefile>

=item B<OUT: >Hashreference to width, height and size of the image

=back

=cut



sub image_properties {
    my $InputImage = shift ;

    my $Image = new Image::Magick;
    
    my $Status = $Image->Read("$InputImage") ;
    print_error($Status) if $Status ;
    
    my ($width,$height,$filesize) = $Image->Get('width','height','filesize') ;

    my $Properties = { width => $width,
		       height => $height,
		       size => $filesize,
		       ksize => int($filesize/1024)} ;
    
    return $Properties ;

}











####################################################################
# match subroutine:

=pod

=head2 match

This subroutine checks for every element of a given array if it matches (or not matches) a given RegExp.

=over 7

=item B<IN : ><(!)RegExp to match>, <Array to match>

If the first letter of the RegExp is a '!' the match subroutine
returns all entries that do NOT match

=item B<OUT: >Array of all matched elements

=back

=cut


sub match {
    my $RegExp = shift ;
    my @ToMatch = @_ ;

    my @Matched ;


    if ( $RegExp =~ s/^!// ) {
	print "Negativ Match $RegExp !\n" if $Aww::verbose > 2 ;
	foreach ( @ToMatch ) {
	    push @Matched, $_ if ( $_ !~ /$RegExp/ );

	}
    }
    else {

	foreach ( @ToMatch ) {
	    push @Matched, $_ if ( $_ =~ /$RegExp/ );
	}	    
    }

    return @Matched ;
}





####################################################################
# optioninput subroutine, to add key-valuepairs to a hash
# Input: 1. Hashname and 2. new key
# The value for the new key is <STDIN> (chomped and checked...)

=pod

=head2 optioninput

This subroutine adds new key=value pairs to the project configfile

=over 7

=item B<IN : ><directory>

=back

The subroutine then asks for the options of the directory, chomps of
the linebreak, applies some sanity checks and hands it to the config
subroutine

=over 7

=item B<OUT: >New elements in the project configfile [Options]

=back

=cut


sub optioninput {
    my $NewFile = shift ;
    my $NewDir = shift ;
    my $DirOptions ;
    my @DirOptions ;

    chomp ($DirOptions = <STDIN>);
    print "\n";

    # Check if Options are OK

    @DirOptions = split / /, $DirOptions if $DirOptions ;

    checkoptions(@DirOptions) if @DirOptions;


    # Set Option

    my $OptionList = $DirOptions ;
    $OptionList =~ tr/ /,/ ;

    $NewDir = "Options.$NewDir" ;


    config($NewFile,$NewDir,$OptionList) ;


}






####################################################################
# pft2oft subroutine, to change the suffix
# Input: Processfilename 
# Output: Outputfilename


=pod

=head2 pft2oft

This subroutine changes the suffix of the given Processfile to the Outputfile_type. 

=over 7

=item B<IN : >Filename

=item B<OUT: >Filename with new suffix

=back

=cut


sub pft2oft {
    my $ProcessFilename = shift ;

    my $PFT = config("config","Processfile_type") ;
    my $OFT = config("config","Outputfile_type") ;

    $ProcessFilename =~ s/$PFT/eval "\"$OFT\""/e ;


    return $ProcessFilename ;

}











####################################################################
# print_error function. Returns Error and Usage of aww

=pod

=head2 print_error

This subroutine prints the error-message (IN) and a short help for aww

=over 7

=item B<IN : >[Error message]

If no error message is given, this function only prints a short help

=item B<OUT: >*** ERROR message and a short help -> exit 1 as well

=back

=cut


sub print_error {

my $Usage = <<'_USAGE';

USAGE: 
       aww [-n|-s|-d|-p|-e|-h|-v] <name of your project>

            -n: create a new aww-project
            -s: show a given aww-project and all wml options
            -d: delete a aww-project
            -f: force the processing of all sourcefiles
            -p: process a aww-project (same as no option)
            -e: export HTML-Output of an wml-website
            -h: print this short help
	    -v: verbose output

_USAGE

print "\n   *** ERROR: @_ ***\n\n" if @_ ;
print "   aww = (a)dminister (w)ml (w)ebsites
   a frontend to administer your homepages written 
   with wml the Website Meta language              \n
               Version: $Aww::VERSION\n" unless @_ ;

print $Usage;
exit 1;

}






####################################################################
# process subroutine
# Input: FullSrcfile, FullDestFile, wml-options

=pod

=head2 process

This is the core processing subroutine, that directly executes wml

=over 7

=item B<IN : ><FullSourceFile>, <FullDestFile>, [wml-options]

=item B<OUT: >wml -o $FullDestFile @WMLOpts $FullSrcFile

=back

=cut

sub process {
    my $FullSrcFile = shift ;
    my $FullDestFile = shift ;
    my @WMLOpts = @_ ;

    print "Executing: wml -o $FullDestFile @WMLOpts $FullSrcFile \n"  if $Aww::verbose > 2 ;
    qx(wml -o $FullDestFile @WMLOpts $FullSrcFile)
}






####################################################################
# trashfiles subroutine:
# In: File to delete later, 
# or _CLEAR to delete all collected Files up to now

=pod

=head2 trashfiles

This subroutine collects files that should be deleted befor the
program terminates.

=over 7

=item B<IN : >< file to delete | _CLEAR >

=item B<OUT: >deleting files when a _CLEAR is send (e.g. before the
end of the program

=back

=cut

sub trashfiles {
    my $TrashFile = shift ;

    our @FilesToDelete ;
    if ( -w $TrashFile ) {
	push @FilesToDelete, $TrashFile ;
    }
    elsif ( $TrashFile =~ /^_CLEAR$/ ) {
	unlink @FilesToDelete ;
	print "unlinking Files: @FilesToDelete\n" if $Aww::verbose > 1 ;
	undef @FilesToDelete ;
    }
    else {
	die "Wrong Input ($TrashFile) for trashfile-subroutine!\n" ;
    }

    return ;

}








####################################################################
# wmlprocess subroutine
# Input: Directory to process

=pod

=head2 wmlprocess

This is the main processing subroutine. It checks for newer
sourcefiles in the given sourcedirectory and processes all needed
files.

=over 7

=item B<IN : ><directory>

=back

=cut

sub wmlprocess {
    my $SrcDir = shift ;
    my @ProcessFiles ;
    print "\nWML-Processing of Directory: '$SrcDir'\n\n" if $Aww::verbose > 1 ;

    # get Files from the Src Directory

    my $MatchOption = config("config","Processfile_type") ;
    $MatchOption = "$MatchOption\$" ;

    my @SrcFiles = get_fullfilenames($SrcDir, "none" , $MatchOption) ;
    print "Files in Source-Dir: @SrcFiles \n" if $Aww::verbose > 1 ;

    # check for newer source files

    @ProcessFiles = checktime(@SrcFiles) ;


    foreach my $ProcessSrc ( @ProcessFiles ) {
	my $ProcessDest = $ProcessSrc ;

	$ProcessDest =~ s/^$Aww::Main::ProcessVars->{'Main.Src_Base_Directory'}/$Aww::Main::ProcessVars->{'Main.Output_Base_Directory'}/ ;

	$ProcessDest = pft2oft($ProcessDest) ;

	process ($ProcessSrc,$ProcessDest);
    }



}





####################################################################
# writemainonfig function

=pod

=head2 writemainconfig

This subroutine writes the main configfile to the $HOME/.aww/ directory

=cut


sub writemainconfig {


# Create new config file


my $ConfigFile = <<'_CONFIG';
###################################################
#      This is the default aww config-file        #
###################################################



# The filetypes used with your wml website
# (only one ending per type possible!)

Processfile_type = '\.(s|)wml' 

Outputfile_type = '\.$1html' 

Includefile_type = '.inc' 

Templatefile_type = '.tmpl' 

WMLRCFile = '.wmlrc' 


# The program you use for exporting your websites
# at this time only 'lftp' and 'ncftpput' is supported

Export_FTP_Program = 'lftp' 


# Set this to a maximum value of 3 for full verbosity

Verbose = '0' 




###################################################
# You shouldn't change any options below this line 
# unless you really know what you are doing!      


# The version-variable: used for update procedures only

MinVersion = '0.3' 


# The possible options that can be used with aww 
# (see Aww::Options Module)

PossibleOptions = 'index','thumbnail','duplicate','english'





###################################################
#          End of default .aww/config             # 
###################################################

_CONFIG

    mkdir "$ENV{HOME}/.aww" ;

    open (CONFIGURATION, ">$ENV{HOME}/.aww/config") or die "***ERROR*** Couldn't create new config file: $ENV{HOME}/.aww/config " ;
    print CONFIGURATION $ConfigFile ;
    close CONFIGURATION ;

    print "Writing new config file: $ENV{HOME}/.aww/config \n"; 
    print "Please edit the file for your needs! --> Exiting!\n"; 
    exit 1;

}


















1;
__END__





=pod

=head1 AUTHOR

Michael Panhorst <aww[at]panhorst.com>

=head1 SEE ALSO

B<L<aww>>, B<L<Aww::Main>> and B<L<Aww::Options>>

=head1 BUGS

None known.

=cut
